import { Bounded } from './Bounded';
import { Endomorphism } from './function';
import { Semigroup } from './Semigroup';
/**
 * @typeclass
 * @since 1.0.0
 */
export interface Monoid<A> extends Semigroup<A> {
    readonly empty: A;
}
/**
 * @function
 * @since 1.0.0
 */
export declare const fold: <A>(M: Monoid<A>) => (as: A[]) => A;
/**
 * @function
 * @since 1.0.0
 */
export declare const getProductMonoid: <A, B>(MA: Monoid<A>, MB: Monoid<B>) => Monoid<[A, B]>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getDualMonoid: <A>(M: Monoid<A>) => Monoid<A>;
/**
 * Boolean monoid under conjunction
 * @instance
 * @since 1.0.0
 */
export declare const monoidAll: Monoid<boolean>;
/**
 * Boolean monoid under disjunction
 * @instance
 * @since 1.0.0
 */
export declare const monoidAny: Monoid<boolean>;
/**
 * @instance
 * @since 1.0.0
 */
export declare const unsafeMonoidArray: Monoid<Array<any>>;
/**
 * Monoid under array concatenation (`Array<any>`)
 * @function
 * @since 1.0.0
 */
export declare const getArrayMonoid: <A = never>() => Monoid<A[]>;
/**
 * Gets {@link Monoid} instance for dictionaries given {@link Semigroup} instance for their values
 *
 * @example
 * import { getDictionaryMonoid, fold } from 'fp-ts/lib/Monoid'
 * import { semigroupSum } from 'fp-ts/lib/Semigroup'
 *
 * const M = getDictionaryMonoid(semigroupSum)
 * assert.deepEqual(fold(M)([{ foo: 123 }, { foo: 456 }]), { foo: 579 })
 *
 * @function
 * @since 1.4.0
 */
export declare const getDictionaryMonoid: <A>(S: Semigroup<A>) => Monoid<{
    [key: string]: A;
}>;
/**
 * Number monoid under addition
 * @instance
 * @since 1.0.0
 */
export declare const monoidSum: Monoid<number>;
/**
 * Number monoid under multiplication
 * @instance
 * @since 1.0.0
 */
export declare const monoidProduct: Monoid<number>;
/**
 * @instance
 * @since 1.0.0
 */
export declare const monoidString: Monoid<string>;
/**
 * @instance
 * @since 1.0.0
 */
export declare const monoidVoid: Monoid<void>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getFunctionMonoid: <M>(M: Monoid<M>) => <A = never>() => Monoid<(a: A) => M>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getEndomorphismMonoid: <A = never>() => Monoid<Endomorphism<A>>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getRecordMonoid: <O extends {
    [key: string]: any;
}>(monoids: { [K in keyof O]: Monoid<O[K]>; }) => Monoid<O>;
/**
 * @function
 * @since 1.9.0
 */
export declare const getMeetMonoid: <A>(B: Bounded<A>) => Monoid<A>;
/**
 * @function
 * @since 1.9.0
 */
export declare const getJoinMonoid: <A>(B: Bounded<A>) => Monoid<A>;
