import { Ord } from './Ord';
/**
 * @typeclass
 * @since 1.0.0
 */
export interface Semigroup<A> {
    readonly concat: (x: A, y: A) => A;
}
/**
 * @function
 * @since 1.0.0
 */
export declare const fold: <A>(S: Semigroup<A>) => (a: A) => (as: A[]) => A;
/**
 * @function
 * @since 1.0.0
 */
export declare const getFirstSemigroup: <A = never>() => Semigroup<A>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getLastSemigroup: <A = never>() => Semigroup<A>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getProductSemigroup: <A, B>(SA: Semigroup<A>, SB: Semigroup<B>) => Semigroup<[A, B]>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getDualSemigroup: <A>(S: Semigroup<A>) => Semigroup<A>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getFunctionSemigroup: <S>(S: Semigroup<S>) => <A = never>() => Semigroup<(a: A) => S>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getRecordSemigroup: <O extends {
    [key: string]: any;
}>(semigroups: { [K in keyof O]: Semigroup<O[K]>; }) => Semigroup<O>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getMeetSemigroup: <A>(O: Ord<A>) => Semigroup<A>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getJoinSemigroup: <A>(O: Ord<A>) => Semigroup<A>;
/**
 * Boolean semigroup under conjunction
 * @instance
 * @since 1.0.0
 */
export declare const semigroupAll: Semigroup<boolean>;
/**
 * Boolean semigroup under disjunction
 * @instance
 * @since 1.0.0
 */
export declare const semigroupAny: Semigroup<boolean>;
/**
 * Semigroup under array concatenation
 * @function
 * @since 1.0.0
 */
export declare const getArraySemigroup: <A = never>() => Semigroup<A[]>;
/**
 * Gets {@link Semigroup} instance for dictionaries given {@link Semigroup} instance for their values
 *
 * @example
 * import { getDictionarySemigroup, semigroupSum } from 'fp-ts/lib/Semigroup'
 *
 * const S = getDictionarySemigroup(semigroupSum)
 * assert.deepEqual(S.concat({ foo: 123 }, { foo: 456 }), { foo: 579 })
 *
 * @function
 * @since 1.4.0
 */
export declare const getDictionarySemigroup: <A>(S: Semigroup<A>) => Semigroup<{
    [key: string]: A;
}>;
/**
 * Gets {@link Semigroup} instance for objects of given type preserving their type
 *
 * @example
 * import { getObjectSemigroup } from 'fp-ts/lib/Semigroup'
 *
 * const S = getObjectSemigroup<{ foo: number }>()
 * assert.deepEqual(S.concat({ foo: 123 }, { foo: 456 }), { foo: 456 })
 *
 * @function
 * @since 1.4.0
 */
export declare const getObjectSemigroup: <A extends object = never>() => Semigroup<A>;
/**
 * Number Semigroup under addition
 * @instance
 * @since 1.0.0
 */
export declare const semigroupSum: Semigroup<number>;
/**
 * Number Semigroup under multiplication
 * @instance
 * @since 1.0.0
 */
export declare const semigroupProduct: Semigroup<number>;
/**
 * @instance
 * @since 1.0.0
 */
export declare const semigroupString: Semigroup<string>;
/**
 * @instance
 * @since 1.0.0
 */
export declare const semigroupVoid: Semigroup<void>;
