/**
 * The `Setoid` type class represents types which support decidable equality.
 *
 * Instances must satisfy the following laws:
 *
 * 1. Reflexivity: `S.equals(a, a) === true`
 * 2. Symmetry: `S.equals(a, b) === S.equals(b, a)`
 * 3. Transitivity: if `S.equals(a, b) === true` and `S.equals(b, c) === true`, then `S.equals(a, c) === true`
 *
 * @typeclass
 * @since 1.0.0
 */
export interface Setoid<A> {
    readonly equals: (x: A, y: A) => boolean;
}
/**
 * @function
 * @since 1.0.0
 */
export declare const strictEqual: <A>(a: A, b: A) => boolean;
/**
 * @instance
 * @since 1.0.0
 */
export declare const setoidString: Setoid<string>;
/**
 * @instance
 * @since 1.0.0
 */
export declare const setoidNumber: Setoid<number>;
/**
 * @instance
 * @since 1.0.0
 */
export declare const setoidBoolean: Setoid<boolean>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getArraySetoid: <A>(S: Setoid<A>) => Setoid<A[]>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getRecordSetoid: <O extends {
    [key: string]: any;
}>(setoids: { [K in keyof O]: Setoid<O[K]>; }) => Setoid<O>;
/**
 * @function
 * @since 1.0.0
 */
export declare const getProductSetoid: <A, B>(SA: Setoid<A>, SB: Setoid<B>) => Setoid<[A, B]>;
/**
 * Returns the `Setoid` corresponding to the partitions of `B` induced by `f`
 * @function
 * @since 1.2.0
 */
export declare const contramap: <A, B>(f: (b: B) => A, fa: Setoid<A>) => Setoid<B>;
/**
 * @instance
 * @since 1.4.0
 */
export declare const setoidDate: Setoid<Date>;
